@@ -22,7 +22,7 @@
 Custom progress classes should inherit from these classes. They can also be
 used as dummy progress classes which simply do nothing.
 """
-from __future__ import print_function
+from __future__ import annotations
 
 import errno
 import fcntl
@@ -32,38 +32,31 @@ import re
 import select
 import sys
 
-from typing import Optional, Union
-
 import apt_pkg
 
-__all__ = ['AcquireProgress', 'CdromProgress', 'InstallProgress', 'OpProgress']
+__all__ = ["AcquireProgress", "CdromProgress", "InstallProgress", "OpProgress"]
 
 
-class AcquireProgress(object):
+class AcquireProgress:
     """Monitor object for downloads controlled by the Acquire class.
 
     This is an mostly abstract class. You should subclass it and implement the
     methods to get something useful.
     """
 
-    current_bytes = current_cps = fetched_bytes = last_bytes = total_bytes \
-                  = 0.0
+    current_bytes = current_cps = fetched_bytes = last_bytes = total_bytes = 0.0
     current_items = elapsed_time = total_items = 0
 
-    def done(self, item):
-        # type: (apt_pkg.AcquireItemDesc) -> None
+    def done(self, item: apt_pkg.AcquireItemDesc) -> None:
         """Invoked when an item is successfully and completely fetched."""
 
-    def fail(self, item):
-        # type: (apt_pkg.AcquireItemDesc) -> None
+    def fail(self, item: apt_pkg.AcquireItemDesc) -> None:
         """Invoked when an item could not be fetched."""
 
-    def fetch(self, item):
-        # type: (apt_pkg.AcquireItemDesc) -> None
+    def fetch(self, item: apt_pkg.AcquireItemDesc) -> None:
         """Invoked when some of the item's data is fetched."""
 
-    def ims_hit(self, item):
-        # type: (apt_pkg.AcquireItemDesc) -> None
+    def ims_hit(self, item: apt_pkg.AcquireItemDesc) -> None:
         """Invoked when an item is confirmed to be up-to-date.
 
         Invoked when an item is confirmed to be up-to-date. For instance,
@@ -71,8 +64,7 @@ class AcquireProgress(object):
         not modified.
         """
 
-    def media_change(self, media, drive):
-        # type: (str, str) -> bool
+    def media_change(self, media: str, drive: str) -> bool:
         """Prompt the user to change the inserted removable media.
 
         The parameter 'media' decribes the name of the media type that
@@ -85,8 +77,7 @@ class AcquireProgress(object):
         """
         return False
 
-    def pulse(self, owner):
-        # type: (apt_pkg.Acquire) -> bool
+    def pulse(self, owner: apt_pkg.Acquire) -> bool:
         """Periodically invoked while the Acquire process is underway.
 
         This method gets invoked while the Acquire progress given by the
@@ -98,8 +89,7 @@ class AcquireProgress(object):
         """
         return True
 
-    def start(self):
-        # type: () -> None
+    def start(self) -> None:
         """Invoked when the Acquire process starts running."""
         # Reset all our values.
         self.current_bytes = 0.0
@@ -111,12 +101,11 @@ class AcquireProgress(object):
         self.total_bytes = 0.0
         self.total_items = 0
 
-    def stop(self):
-        # type: () -> None
+    def stop(self) -> None:
         """Invoked when the Acquire process stops running."""
 
 
-class CdromProgress(object):
+class CdromProgress:
     """Base class for reporting the progress of adding a cdrom.
 
     Can be used with apt_pkg.Cdrom to produce an utility like apt-cdrom. The
@@ -126,24 +115,22 @@ class CdromProgress(object):
 
     total_steps = 0
 
-    def ask_cdrom_name(self):
-        # type: () -> Optional[str]
+    def ask_cdrom_name(self) -> str | None:
         """Ask for the name of the cdrom.
 
         If a name has been provided, return it. Otherwise, return None to
         cancel the operation.
         """
 
-    def change_cdrom(self):
-        # type: () -> bool
+    def change_cdrom(self) -> bool:
         """Ask for the CD-ROM to be changed.
 
         Return True once the cdrom has been changed or False to cancel the
         operation.
         """
+        return False
 
-    def update(self, text, current):
-        # type: (str, int) -> None
+    def update(self, text: str, current: int) -> None:
         """Periodically invoked to update the interface.
 
         The string 'text' defines the text which should be displayed. The
@@ -151,54 +138,44 @@ class CdromProgress(object):
         """
 
 
-class InstallProgress(object):
+class InstallProgress:
     """Class to report the progress of installing packages."""
 
     child_pid, percent, select_timeout, status = 0, 0.0, 0.1, ""
 
-    def __init__(self):
-        # type: () -> None
+    def __init__(self) -> None:
         (self.statusfd, self.writefd) = os.pipe()
         # These will leak fds, but fixing this safely requires API changes.
-        self.write_stream = os.fdopen(self.writefd, "w")  # type: io.TextIOBase
-        self.status_stream = os.fdopen(self.statusfd, "r")  # type: io.TextIOBase # noqa
+        self.write_stream: io.TextIOBase = os.fdopen(self.writefd, "w")
+        self.status_stream: io.TextIOBase = os.fdopen(self.statusfd, "r")  # noqa
         fcntl.fcntl(self.statusfd, fcntl.F_SETFL, os.O_NONBLOCK)
 
-    def start_update(self):
-        # type: () -> None
+    def start_update(self) -> None:
         """(Abstract) Start update."""
 
-    def finish_update(self):
-        # type: () -> None
+    def finish_update(self) -> None:
         """(Abstract) Called when update has finished."""
 
-    def __enter__(self):
-        # type: () -> InstallProgress
+    def __enter__(self) -> InstallProgress:
         return self
 
-    def __exit__(self, type, value, traceback):
-        # type: (object, object, object) -> None
+    def __exit__(self, type: object, value: object, traceback: object) -> None:
         self.write_stream.close()
         self.status_stream.close()
 
-    def error(self, pkg, errormsg):
-        # type: (str, str) -> None
+    def error(self, pkg: str, errormsg: str) -> None:
         """(Abstract) Called when a error is detected during the install."""
 
-    def conffile(self, current, new):
-        # type: (str, str) -> None
+    def conffile(self, current: str, new: str) -> None:
         """(Abstract) Called when a conffile question from dpkg is detected."""
 
-    def status_change(self, pkg, percent, status):
-        # type: (str, float, str) -> None
+    def status_change(self, pkg: str, percent: float, status: str) -> None:
         """(Abstract) Called when the APT status changed."""
 
-    def dpkg_status_change(self, pkg, status):
-        # type: (str, str) -> None
+    def dpkg_status_change(self, pkg: str, status: str) -> None:
         """(Abstract) Called when the dpkg status changed."""
 
-    def processing(self, pkg, stage):
-        # type: (str, str) -> None
+    def processing(self, pkg: str, stage: str) -> None:
         """(Abstract) Sent just before a processing stage starts.
 
         The parameter 'stage' is one of "upgrade", "install"
@@ -206,8 +183,7 @@ class InstallProgress(object):
         "purge". This method is used for dpkg only.
         """
 
-    def run(self, obj):
-        # type: (Union[apt_pkg.PackageManager, Union[bytes, str]]) -> int
+    def run(self, obj: apt_pkg.PackageManager | bytes | str) -> int:
         """Install using the object 'obj'.
 
         This functions runs install actions. The parameter 'obj' may either
@@ -236,9 +212,17 @@ class InstallProgress(object):
             try:
                 os._exit(obj.do_install(self.write_stream.fileno()))  # type: ignore # noqa
             except AttributeError:
-                os._exit(os.spawnlp(os.P_WAIT, "dpkg", "dpkg", "--status-fd",
-                                    str(self.write_stream.fileno()), "-i",
-                                    obj))  # type: ignore # noqa
+                os._exit(
+                    os.spawnlp(
+                        os.P_WAIT,
+                        "dpkg",
+                        "dpkg",
+                        "--status-fd",
+                        str(self.write_stream.fileno()),
+                        "-i",
+                        obj,  # type: ignore # noqa
+                    )
+                )
             except Exception as e:
                 sys.stderr.write("%s\n" % e)
                 os._exit(apt_pkg.PackageManager.RESULT_FAILED)
@@ -247,17 +231,15 @@ class InstallProgress(object):
         res = self.wait_child()
         return os.WEXITSTATUS(res)
 
-    def fork(self):
-        # type: () -> int
+    def fork(self) -> int:
         """Fork."""
         return os.fork()
 
-    def update_interface(self):
-        # type: () -> None
+    def update_interface(self) -> None:
         """Update the interface."""
         try:
             line = self.status_stream.readline()
-        except IOError as err:
+        except OSError as err:
             # resource temporarly unavailable is ignored
             if err.errno != errno.EAGAIN and err.errno != errno.EWOULDBLOCK:
                 print(err.strerror)
@@ -265,18 +247,18 @@ class InstallProgress(object):
 
         pkgname = status = status_str = percent = base = ""
 
-        if line.startswith('pm'):
+        if line.startswith("pm"):
             try:
                 (status, pkgname, percent, status_str) = line.split(":", 3)
             except ValueError:
                 # silently ignore lines that can't be parsed
                 return
-        elif line.startswith('status'):
+        elif line.startswith("status"):
             try:
                 (base, pkgname, status, status_str) = line.split(":", 3)
             except ValueError:
                 (base, pkgname, status) = line.split(":", 2)
-        elif line.startswith('processing'):
+        elif line.startswith("processing"):
             (status, status_str, pkgname) = line.split(":", 2)
             self.processing(pkgname.strip(), status_str.strip())
 
@@ -285,10 +267,10 @@ class InstallProgress(object):
         status_str = status_str.strip()
         status = status.strip()
 
-        if status == 'pmerror' or status == 'error':
+        if status == "pmerror" or status == "error":
             self.error(pkgname, status_str)
-        elif status == 'conffile-prompt' or status == 'pmconffile':
-            match = re.match("\\s*\'(.*)\'\\s*\'(.*)\'.*", status_str)
+        elif status == "conffile-prompt" or status == "pmconffile":
+            match = re.match("\\s*'(.*)'\\s*'(.*)'.*", status_str)
             if match:
                 self.conffile(match.group(1), match.group(2))
         elif status == "pmstatus":
@@ -300,8 +282,7 @@ class InstallProgress(object):
         elif base == "status":
             self.dpkg_status_change(pkgname, status)
 
-    def wait_child(self):
-        # type: () -> int
+    def wait_child(self) -> int:
         """Wait for child progress to exit.
 
         This method is responsible for calling update_interface() from time to
@@ -311,9 +292,8 @@ class InstallProgress(object):
         (pid, res) = (0, 0)
         while True:
             try:
-                select.select([self.status_stream], [], [],
-                              self.select_timeout)
-            except select.error as error:
+                select.select([self.status_stream], [], [], self.select_timeout)
+            except OSError as error:
                 (errno_, _errstr) = error.args
                 if errno_ != errno.EINTR:
                     raise
@@ -332,15 +312,14 @@ class InstallProgress(object):
         return res
 
 
-class OpProgress(object):
+class OpProgress:
     """Monitor objects for operations.
 
     Display the progress of operations such as opening the cache."""
 
     major_change, op, percent, subop = False, "", 0.0, ""
 
-    def update(self, percent=None):
-        # type: (Optional[float]) -> None
+    def update(self, percent: float | None = None) -> None:
         """Called periodically to update the user interface.
 
         You may use the optional argument 'percent' to set the attribute
@@ -349,6 +328,5 @@ class OpProgress(object):
         if percent is not None:
             self.percent = percent
 
-    def done(self):
-        # type: () -> None
+    def done(self) -> None:
         """Called once an operation has been completed."""
