@@ -1,24 +1,24 @@
 # This file is part of cloud-init. See LICENSE file for license information.
+
+# TODO: Importing this file without first importing
+# cloudinit.sources.azure.errors will result in a circular import.
 import base64
 import json
 import logging
 import os
 import re
-import socket
-import struct
 import textwrap
-import time
 import zlib
 from contextlib import contextmanager
-from datetime import datetime
-from errno import ENOENT
-from typing import List, Optional
-from xml.etree import ElementTree
-from xml.sax.saxutils import escape
+from datetime import datetime, timezone
+from time import sleep, time
+from typing import Callable, List, Optional, TypeVar, Union
+from xml.etree import ElementTree as ET  # nosec B405
+from xml.sax.saxutils import escape  # nosec B406
 
 from cloudinit import distros, subp, temp_utils, url_helper, util, version
 from cloudinit.reporting import events
-from cloudinit.settings import CFG_BUILTIN
+from cloudinit.sources.azure import errors
 
 LOG = logging.getLogger(__name__)
 
@@ -29,28 +29,16 @@ BOOT_EVENT_TYPE = "boot-telemetry"
 SYSTEMINFO_EVENT_TYPE = "system-info"
 DIAGNOSTIC_EVENT_TYPE = "diagnostic"
 COMPRESSED_EVENT_TYPE = "compressed"
-# Maximum number of bytes of the cloud-init.log file that can be dumped to KVP
-# at once. This number is based on the analysis done on a large sample of
-# cloud-init.log files where the P95 of the file sizes was 537KB and the time
-# consumed to dump 500KB file was (P95:76, P99:233, P99.9:1170) in ms
-MAX_LOG_TO_KVP_LENGTH = 512000
-# File to store the last byte of cloud-init.log that was pushed to KVP. This
-# file will be deleted with every VM reboot.
-LOG_PUSHED_TO_KVP_INDEX_FILE = "/run/cloud-init/log_pushed_to_kvp_index"
 azure_ds_reporter = events.ReportEventStack(
     name="azure-ds",
     description="initialize reporter for azure ds",
     reporting_enabled=True,
 )
 
-DEFAULT_REPORT_FAILURE_USER_VISIBLE_MESSAGE = (
-    "The VM encountered an error during deployment. "
-    "Please visit https://aka.ms/linuxprovisioningerror "
-    "for more information on remediation."
-)
+T = TypeVar("T")
 
 
-def azure_ds_telemetry_reporter(func):
+def azure_ds_telemetry_reporter(func: Callable[..., T]) -> Callable[..., T]:
     def impl(*args, **kwargs):
         with events.ReportEventStack(
             name=func.__name__,
@@ -62,34 +50,6 @@ def azure_ds_telemetry_reporter(func):
     return impl
 
 
-def is_byte_swapped(previous_id, current_id):
-    """
-    Azure stores the instance ID with an incorrect byte ordering for the
-    first parts. This corrects the byte order such that it is consistent with
-    that returned by the metadata service.
-    """
-    if previous_id == current_id:
-        return False
-
-    def swap_bytestring(s, width=2):
-        dd = [byte for byte in textwrap.wrap(s, 2)]
-        dd.reverse()
-        return "".join(dd)
-
-    parts = current_id.split("-")
-    swapped_id = "-".join(
-        [
-            swap_bytestring(parts[0]),
-            swap_bytestring(parts[1]),
-            swap_bytestring(parts[2]),
-            parts[3],
-            parts[4],
-        ]
-    )
-
-    return previous_id == swapped_id
-
-
 @azure_ds_telemetry_reporter
 def get_boot_telemetry():
     """Report timestamps related to kernel initialization and systemd
@@ -99,7 +59,7 @@ def get_boot_telemetry():
 
     LOG.debug("Collecting boot telemetry")
     try:
-        kernel_start = float(time.time()) - float(util.uptime())
+        kernel_start = float(time()) - float(util.uptime())
     except ValueError as e:
         raise RuntimeError("Failed to determine kernel start timestamp") from e
 
@@ -162,9 +122,11 @@ def get_boot_telemetry():
         "boot-telemetry",
         "kernel_start=%s user_start=%s cloudinit_activation=%s"
         % (
-            datetime.utcfromtimestamp(kernel_start).isoformat() + "Z",
-            datetime.utcfromtimestamp(user_start).isoformat() + "Z",
-            datetime.utcfromtimestamp(cloudinit_activation).isoformat() + "Z",
+            datetime.fromtimestamp(kernel_start, timezone.utc).isoformat(),
+            datetime.fromtimestamp(user_start, timezone.utc).isoformat(),
+            datetime.fromtimestamp(
+                cloudinit_activation, timezone.utc
+            ).isoformat(),
         ),
         events.DEFAULT_EVENT_ORIGIN,
     )
@@ -241,35 +203,8 @@ def report_compressed_event(event_name, event_content):
 
 
 @azure_ds_telemetry_reporter
-def push_log_to_kvp(file_name=CFG_BUILTIN["def_log_file"]):
-    """Push a portion of cloud-init.log file or the whole file to KVP
-    based on the file size.
-    The first time this function is called after VM boot, It will push the last
-    n bytes of the log file such that n < MAX_LOG_TO_KVP_LENGTH
-    If called again on the same boot, it continues from where it left off.
-    In addition to cloud-init.log, dmesg log will also be collected."""
-
-    start_index = get_last_log_byte_pushed_to_kvp_index()
-
-    LOG.debug("Dumping cloud-init.log file to KVP")
-    try:
-        with open(file_name, "rb") as f:
-            f.seek(0, os.SEEK_END)
-            seek_index = max(f.tell() - MAX_LOG_TO_KVP_LENGTH, start_index)
-            report_diagnostic_event(
-                "Dumping last {0} bytes of cloud-init.log file to KVP starting"
-                " from index: {1}".format(f.tell() - seek_index, seek_index),
-                logger_func=LOG.debug,
-            )
-            f.seek(seek_index, os.SEEK_SET)
-            report_compressed_event("cloud-init.log", f.read())
-            util.write_file(LOG_PUSHED_TO_KVP_INDEX_FILE, str(f.tell()))
-    except Exception as ex:
-        report_diagnostic_event(
-            "Exception when dumping log file: %s" % repr(ex),
-            logger_func=LOG.warning,
-        )
-
+def report_dmesg_to_kvp():
+    """Report dmesg to KVP."""
     LOG.debug("Dumping dmesg log to KVP")
     try:
         out, _ = subp.subp(["dmesg"], decode=False, capture=True)
@@ -281,30 +216,6 @@ def push_log_to_kvp(file_name=CFG_BUILTIN["def_log_file"]):
         )
 
 
-@azure_ds_telemetry_reporter
-def get_last_log_byte_pushed_to_kvp_index():
-    try:
-        with open(LOG_PUSHED_TO_KVP_INDEX_FILE, "r") as f:
-            return int(f.read())
-    except IOError as e:
-        if e.errno != ENOENT:
-            report_diagnostic_event(
-                "Reading LOG_PUSHED_TO_KVP_INDEX_FILE failed: %s." % repr(e),
-                logger_func=LOG.warning,
-            )
-    except ValueError as e:
-        report_diagnostic_event(
-            "Invalid value in LOG_PUSHED_TO_KVP_INDEX_FILE: %s." % repr(e),
-            logger_func=LOG.warning,
-        )
-    except Exception as e:
-        report_diagnostic_event(
-            "Failed to get the last log byte pushed to KVP: %s." % repr(e),
-            logger_func=LOG.warning,
-        )
-    return 0
-
-
 @contextmanager
 def cd(newdir):
     prevdir = os.getcwd()
@@ -315,61 +226,59 @@ def cd(newdir):
         os.chdir(prevdir)
 
 
-def get_ip_from_lease_value(fallback_lease_value):
-    unescaped_value = fallback_lease_value.replace("\\", "")
-    if len(unescaped_value) > 4:
-        hex_string = ""
-        for hex_pair in unescaped_value.split(":"):
-            if len(hex_pair) == 1:
-                hex_pair = "0" + hex_pair
-            hex_string += hex_pair
-        packed_bytes = struct.pack(">L", int(hex_string.replace(":", ""), 16))
-    else:
-        packed_bytes = unescaped_value.encode("utf-8")
-    return socket.inet_ntoa(packed_bytes)
-
-
 @azure_ds_telemetry_reporter
 def http_with_retries(
-    url: str, *, headers: dict, data: Optional[str] = None
+    url: str,
+    *,
+    headers: dict,
+    data: Optional[bytes] = None,
+    retry_sleep: int = 5,
+    timeout_minutes: int = 20,
 ) -> url_helper.UrlResponse:
     """Readurl wrapper for querying wireserver.
 
-    Retries up to 40 minutes:
-    240 attempts * (5s timeout + 5s sleep)
+    :param retry_sleep: Time to sleep before retrying.
+    :param timeout_minutes: Retry up to specified number of minutes.
+    :raises UrlError: on error fetching data.
     """
-    max_readurl_attempts = 240
-    readurl_timeout = 5
-    sleep_duration_between_retries = 5
-    periodic_logging_attempts = 12
+    timeout = timeout_minutes * 60 + time()
 
-    for attempt in range(1, max_readurl_attempts + 1):
+    attempt = 0
+    response = None
+    while not response:
+        attempt += 1
         try:
-            ret = url_helper.readurl(
-                url, headers=headers, data=data, timeout=readurl_timeout
+            response = url_helper.readurl(
+                url, headers=headers, data=data, timeout=(5, 60)
             )
-
+            break
+        except url_helper.UrlError as e:
             report_diagnostic_event(
-                "Successful HTTP request with Azure endpoint %s after "
-                "%d attempts" % (url, attempt),
+                "Failed HTTP request with Azure endpoint %s during "
+                "attempt %d with exception: %s (code=%r headers=%r)"
+                % (url, attempt, e, e.code, e.headers),
                 logger_func=LOG.debug,
             )
-
-            return ret
-
-        except Exception as e:
-            if attempt % periodic_logging_attempts == 0:
-                report_diagnostic_event(
-                    "Failed HTTP request with Azure endpoint %s during "
-                    "attempt %d with exception: %s" % (url, attempt, e),
-                    logger_func=LOG.debug,
-                )
-            if attempt == max_readurl_attempts:
+            # Raise exception if we're out of time or network is unreachable.
+            # If network is unreachable:
+            # - retries will not resolve the situation
+            # - for reporting ready for PPS, this generally means VM was put
+            #   to sleep or network interface was unplugged before we see
+            #   the call complete successfully.
+            if (
+                time() + retry_sleep >= timeout
+                or "Network is unreachable" in str(e)
+            ):
                 raise
 
-        time.sleep(sleep_duration_between_retries)
+        sleep(retry_sleep)
 
-    raise RuntimeError("Failed to return in http_with_retries")
+    report_diagnostic_event(
+        "Successful HTTP request with Azure endpoint %s after "
+        "%d attempts" % (url, attempt),
+        logger_func=LOG.debug,
+    )
+    return response
 
 
 def build_minimal_ovf(
@@ -407,7 +316,6 @@ def build_minimal_ovf(
 
 
 class AzureEndpointHttpClient:
-
     headers = {
         "x-ms-agent-name": "WALinuxAgent",
         "x-ms-version": "2012-11-30",
@@ -427,7 +335,7 @@ class AzureEndpointHttpClient:
         return http_with_retries(url, headers=headers)
 
     def post(
-        self, url, data=None, extra_headers=None
+        self, url, data: Optional[bytes] = None, extra_headers=None
     ) -> url_helper.UrlResponse:
         headers = self.headers
         if extra_headers is not None:
@@ -443,7 +351,7 @@ class InvalidGoalStateXMLException(Exception):
 class GoalState:
     def __init__(
         self,
-        unparsed_xml: str,
+        unparsed_xml: Union[str, bytes],
         azure_endpoint_client: AzureEndpointHttpClient,
         need_certificate: bool = True,
     ) -> None:
@@ -457,8 +365,8 @@ class GoalState:
         self.azure_endpoint_client = azure_endpoint_client
 
         try:
-            self.root = ElementTree.fromstring(unparsed_xml)
-        except ElementTree.ParseError as e:
+            self.root = ET.fromstring(unparsed_xml)  # nosec B314
+        except ET.ParseError as e:
             report_diagnostic_event(
                 "Failed to parse GoalState XML: %s" % e,
                 logger_func=LOG.warning,
@@ -504,7 +412,6 @@ class GoalState:
 
 
 class OpenSSLManager:
-
     certificate_names = {
         "private_key": "TransportPrivate.pem",
         "certificate": "TransportCert.pem",
@@ -544,7 +451,7 @@ class OpenSSLManager:
                     "-days",
                     "32768",
                     "-newkey",
-                    "rsa:2048",
+                    "rsa:3072",
                     "-keyout",
                     self.certificate_names["private_key"],
                     "-out",
@@ -552,7 +459,9 @@ class OpenSSLManager:
                 ]
             )
             certificate = ""
-            for line in open(self.certificate_names["certificate"]):
+            for line in util.load_text_file(
+                self.certificate_names["certificate"]
+            ).splitlines():
                 if "CERTIFICATE" not in line:
                     certificate += line.rstrip()
             self.certificate = certificate
@@ -574,7 +483,7 @@ class OpenSSLManager:
 
     @azure_ds_telemetry_reporter
     def _get_fingerprint_from_cert(self, certificate):
-        """openssl x509 formats fingerprints as so:
+        r"""openssl x509 formats fingerprints as so:
         'SHA1 Fingerprint=07:3E:19:D1:4D:1C:79:92:24:C6:A0:FD:8D:DA:\
         B6:A8:BF:27:D4:73\n'
 
@@ -591,7 +500,7 @@ class OpenSSLManager:
         """Decrypt the certificates XML document using the our private key;
         return the list of certs and private keys contained in the doc.
         """
-        tag = ElementTree.fromstring(certificates_xml).find(".//Data")
+        tag = ET.fromstring(certificates_xml).find(".//Data")  # nosec B314
         certificates_content = tag.text
         lines = [
             b"MIME-Version: 1.0",
@@ -633,7 +542,6 @@ class OpenSSLManager:
 
 
 class GoalStateHealthReporter:
-
     HEALTH_REPORT_XML_TEMPLATE = textwrap.dedent(
         """\
         <?xml version="1.0" encoding="utf-8"?>
@@ -739,7 +647,7 @@ class GoalStateHealthReporter:
         status: str,
         substatus=None,
         description=None,
-    ) -> str:
+    ) -> bytes:
         health_detail = ""
         if substatus is not None:
             health_detail = self.HEALTH_DETAIL_SUBSECTION_XML_TEMPLATE.format(
@@ -757,12 +665,10 @@ class GoalStateHealthReporter:
             health_detail_subsection=health_detail,
         )
 
-        return health_report
+        return health_report.encode("utf-8")
 
     @azure_ds_telemetry_reporter
-    def _post_health_report(self, document: str) -> None:
-        push_log_to_kvp()
-
+    def _post_health_report(self, document: bytes) -> None:
         # Whenever report_diagnostic_event(diagnostic_msg) is invoked in code,
         # the diagnostic messages are written to special files
         # (/var/opt/hyperv/.kvp_pool_*) as Hyper-V KVP messages.
@@ -777,11 +683,11 @@ class GoalStateHealthReporter:
         # KVP messages that are published after the Azure Host receives the
         # signal are ignored and unprocessed, so yield this thread to the
         # Hyper-V KVP Reporting thread so that they are written.
-        # time.sleep(0) is a low-cost and proven method to yield the scheduler
+        # sleep(0) is a low-cost and proven method to yield the scheduler
         # and ensure that events are flushed.
         # See HyperVKvpReportingHandler class, which is a multi-threaded
         # reporting handler that writes to the special KVP files.
-        time.sleep(0)
+        sleep(0)
 
         LOG.debug("Sending health report to Azure fabric.")
         url = "http://{}/machine?comp=health".format(self._endpoint)
@@ -804,19 +710,19 @@ class WALinuxAgentShim:
             self.openssl_manager.clean_up()
 
     @azure_ds_telemetry_reporter
-    def eject_iso(self, iso_dev) -> None:
+    def eject_iso(self, iso_dev, distro: distros.Distro) -> None:
+        LOG.debug("Ejecting the provisioning iso")
         try:
-            LOG.debug("Ejecting the provisioning iso")
-            subp.subp(["eject", iso_dev])
+            distro.eject_media(iso_dev)
         except Exception as e:
             report_diagnostic_event(
                 "Failed ejecting the provisioning iso: %s" % e,
-                logger_func=LOG.debug,
+                logger_func=LOG.error,
             )
 
     @azure_ds_telemetry_reporter
     def register_with_azure_and_fetch_data(
-        self, pubkey_info=None, iso_dev=None
+        self, distro: distros.Distro, pubkey_info=None, iso_dev=None
     ) -> Optional[List[str]]:
         """Gets the VM's GoalState from Azure, uses the GoalState information
         to report ready/send the ready signal/provisioning complete signal to
@@ -847,7 +753,7 @@ class WALinuxAgentShim:
         )
 
         if iso_dev is not None:
-            self.eject_iso(iso_dev)
+            self.eject_iso(iso_dev, distro=distro)
 
         health_reporter.send_ready_signal()
         return ssh_keys
@@ -883,7 +789,7 @@ class WALinuxAgentShim:
         )
 
     @azure_ds_telemetry_reporter
-    def _get_raw_goal_state_xml_from_azure(self) -> str:
+    def _get_raw_goal_state_xml_from_azure(self) -> bytes:
         """Fetches the GoalState XML from the Azure endpoint and returns
         the XML as a string.
 
@@ -911,7 +817,9 @@ class WALinuxAgentShim:
 
     @azure_ds_telemetry_reporter
     def _parse_raw_goal_state_xml(
-        self, unparsed_goal_state_xml: str, need_certificate: bool
+        self,
+        unparsed_goal_state_xml: Union[str, bytes],
+        need_certificate: bool,
     ) -> GoalState:
         """Parses a GoalState XML string and returns a GoalState object.
 
@@ -1023,23 +931,23 @@ class WALinuxAgentShim:
 @azure_ds_telemetry_reporter
 def get_metadata_from_fabric(
     endpoint: str,
+    distro: distros.Distro,
     pubkey_info: Optional[List[str]] = None,
     iso_dev: Optional[str] = None,
 ):
     shim = WALinuxAgentShim(endpoint=endpoint)
     try:
         return shim.register_with_azure_and_fetch_data(
-            pubkey_info=pubkey_info, iso_dev=iso_dev
+            distro=distro, pubkey_info=pubkey_info, iso_dev=iso_dev
         )
     finally:
         shim.clean_up()
 
 
 @azure_ds_telemetry_reporter
-def report_failure_to_fabric(endpoint: str, description: Optional[str] = None):
+def report_failure_to_fabric(endpoint: str, error: "errors.ReportableError"):
     shim = WALinuxAgentShim(endpoint=endpoint)
-    if not description:
-        description = DEFAULT_REPORT_FAILURE_USER_VISIBLE_MESSAGE
+    description = error.as_encoded_report()
     try:
         shim.register_with_azure_and_report_failure(description=description)
     finally:
@@ -1055,4 +963,216 @@ def dhcp_log_cb(out, err):
     )
 
 
-# vi: ts=4 expandtab
+class NonAzureDataSource(Exception):
+    pass
+
+
+class OvfEnvXml:
+    NAMESPACES = {
+        "ovf": "http://schemas.dmtf.org/ovf/environment/1",
+        "wa": "http://schemas.microsoft.com/windowsazure",
+    }
+
+    def __init__(
+        self,
+        *,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        hostname: Optional[str] = None,
+        custom_data: Optional[bytes] = None,
+        disable_ssh_password_auth: Optional[bool] = None,
+        public_keys: Optional[List[dict]] = None,
+        preprovisioned_vm: bool = False,
+        preprovisioned_vm_type: Optional[str] = None,
+        provision_guest_proxy_agent: bool = False,
+    ) -> None:
+        self.username = username
+        self.password = password
+        self.hostname = hostname
+        self.custom_data = custom_data
+        self.disable_ssh_password_auth = disable_ssh_password_auth
+        self.public_keys: List[dict] = public_keys or []
+        self.preprovisioned_vm = preprovisioned_vm
+        self.preprovisioned_vm_type = preprovisioned_vm_type
+        self.provision_guest_proxy_agent = provision_guest_proxy_agent
+
+    def __eq__(self, other) -> bool:
+        return self.__dict__ == other.__dict__
+
+    @classmethod
+    def parse_text(cls, ovf_env_xml: str) -> "OvfEnvXml":
+        """Parser for ovf-env.xml data.
+
+        :raises NonAzureDataSource: if XML is not in Azure's format.
+        :raises errors.ReportableErrorOvfParsingException: if XML is
+                unparsable or invalid.
+        """
+        try:
+            root = ET.fromstring(ovf_env_xml)  # nosec B314
+        except ET.ParseError as e:
+            raise errors.ReportableErrorOvfParsingException(exception=e) from e
+
+        # If there's no provisioning section, it's not Azure ovf-env.xml.
+        if root.find("./wa:ProvisioningSection", cls.NAMESPACES) is None:
+            raise NonAzureDataSource(
+                "Ignoring non-Azure ovf-env.xml: ProvisioningSection not found"
+            )
+
+        instance = OvfEnvXml()
+        instance._parse_linux_configuration_set_section(root)
+        instance._parse_platform_settings_section(root)
+
+        return instance
+
+    def _find(
+        self,
+        node,
+        name: str,
+        required: bool,
+        namespace: str = "wa",
+    ):
+        matches = node.findall(
+            "./%s:%s" % (namespace, name), OvfEnvXml.NAMESPACES
+        )
+        if not matches:
+            msg = "missing configuration for %r" % name
+            LOG.debug(msg)
+            if required:
+                raise errors.ReportableErrorOvfInvalidMetadata(msg)
+            return None
+        elif len(matches) > 1:
+            raise errors.ReportableErrorOvfInvalidMetadata(
+                "multiple configuration matches for %r (%d)"
+                % (name, len(matches))
+            )
+
+        return matches[0]
+
+    def _parse_property(
+        self,
+        node,
+        name: str,
+        required: bool,
+        decode_base64: bool = False,
+        parse_bool: bool = False,
+        default=None,
+    ):
+        matches = node.findall("./wa:" + name, OvfEnvXml.NAMESPACES)
+        if not matches:
+            msg = "missing configuration for %r" % name
+            LOG.debug(msg)
+            if required:
+                raise errors.ReportableErrorOvfInvalidMetadata(msg)
+            return default
+
+        if len(matches) > 1:
+            raise errors.ReportableErrorOvfInvalidMetadata(
+                "multiple configuration matches for %r (%d)"
+                % (name, len(matches))
+            )
+
+        value = matches[0].text
+
+        # Empty string may be None.
+        if value is None:
+            value = default
+
+        if decode_base64 and value is not None:
+            value = base64.b64decode("".join(value.split()))
+
+        if parse_bool:
+            value = util.translate_bool(value)
+
+        return value
+
+    def _parse_linux_configuration_set_section(self, root):
+        provisioning_section = self._find(
+            root, "ProvisioningSection", required=True
+        )
+        config_set = self._find(
+            provisioning_section,
+            "LinuxProvisioningConfigurationSet",
+            required=True,
+        )
+
+        self.custom_data = self._parse_property(
+            config_set,
+            "CustomData",
+            decode_base64=True,
+            required=False,
+        )
+        self.username = self._parse_property(
+            config_set, "UserName", required=True
+        )
+        self.password = self._parse_property(
+            config_set, "UserPassword", required=False
+        )
+        self.hostname = self._parse_property(
+            config_set, "HostName", required=True
+        )
+        self.disable_ssh_password_auth = self._parse_property(
+            config_set,
+            "DisableSshPasswordAuthentication",
+            parse_bool=True,
+            required=False,
+        )
+
+        self._parse_ssh_section(config_set)
+
+    def _parse_platform_settings_section(self, root):
+        platform_settings_section = self._find(
+            root, "PlatformSettingsSection", required=True
+        )
+        platform_settings = self._find(
+            platform_settings_section, "PlatformSettings", required=True
+        )
+
+        self.preprovisioned_vm = self._parse_property(
+            platform_settings,
+            "PreprovisionedVm",
+            parse_bool=True,
+            default=False,
+            required=False,
+        )
+        self.preprovisioned_vm_type = self._parse_property(
+            platform_settings,
+            "PreprovisionedVMType",
+            required=False,
+        )
+        self.provision_guest_proxy_agent = self._parse_property(
+            platform_settings,
+            "ProvisionGuestProxyAgent",
+            parse_bool=True,
+            default=False,
+            required=False,
+        )
+
+    def _parse_ssh_section(self, config_set):
+        self.public_keys = []
+
+        ssh_section = self._find(config_set, "SSH", required=False)
+        if ssh_section is None:
+            return
+
+        public_keys_section = self._find(
+            ssh_section, "PublicKeys", required=False
+        )
+        if public_keys_section is None:
+            return
+
+        for public_key in public_keys_section.findall(
+            "./wa:PublicKey", OvfEnvXml.NAMESPACES
+        ):
+            fingerprint = self._parse_property(
+                public_key, "Fingerprint", required=False
+            )
+            path = self._parse_property(public_key, "Path", required=False)
+            value = self._parse_property(
+                public_key, "Value", default="", required=False
+            )
+            ssh_key = {
+                "fingerprint": fingerprint,
+                "path": path,
+                "value": value,
+            }
+            self.public_keys.append(ssh_key)
