@@ -12,6 +12,7 @@ class ApplicationStatus(enum.Enum):
 
     ENABLED = object()
     DISABLED = object()
+    WARNING = object()
 
 
 @enum.unique
@@ -116,8 +117,11 @@ class CanDisableFailureReason(enum.Enum):
     """
 
     ALREADY_DISABLED = object()
+    NOT_APPLICABLE = object()
     ACTIVE_DEPENDENT_SERVICES = object()
+    PURGE_NOT_SUPPORTED = object()
     NOT_FOUND_DEPENDENT_SERVICE = object()
+    NO_PURGE_WITHOUT_ORIGIN = object()
 
 
 class CanDisableFailure:
@@ -128,3 +132,7 @@ class CanDisableFailure:
     ) -> None:
         self.reason = reason
         self.message = message
+
+    @property
+    def message_value(self) -> str:
+        return self.message.msg if self.message else ""
